classdef Boundary
    
    
    properties
        
        nodes
        edges
        types
        parameters
        
    end
    
    
    methods
        
        function obj=Boundary(nodes,edges,type,param)
            % BOUNDARY : Boundary class.
            %
            %
            %       Constructors :
            %
            %       obj=Boundary(nodes) create a Boundary object 
            %       representing a simply connected polygonal domain. nodes
            %       is a 2xN matrix containing the coordinates of the nodes
            %       given in the trigonometric sense.
            %
            %       b=Boundary(nodes,edges) create a Boundary object
            %       representing a polygonal domain. nodes is a 2xN matrix
            %       containing the coordinates of the nodes. edges is 2xM
            %       matrix containing the indices of the starting and
            %       ending nodes of each edge.
            %
            %       b=Boundary(nodes,edges,type,param) create a more
            %       general Boundary object. type is a row containing the
            %       type of each edge. 1 for a segment, 2 for a circle arc
            %       and 3 for an ellipse arc. param is a 7xM matrix, each
            %       column corresponds to an edge filled as follow :
            %       -row 1 contains the left parameter (1 iff the domain 
            %       is on the left)
            %       -row 2 contains the right parameter (1 iff the domain 
            %       is on the right)
            %       -row 3 and 4 contain the coordiate of the center for
            %       the circle arc and the ellipse arc
            %       -row 5 contains the radius of the circle arc or the
            %       first half axe of the ellipse arc
            %       -row 6 contains the second helf axe of the ellipse arc
            %       -row 7 contains the orientation of the ellipse arc.
            %
            %
            %       Methods :
            %
            %       obj.disp display informations contained in the Boundary 
            %       object.
            %
            %       obj.plot draws the Boundary object
            %
            %       gm=obj.geometrymatrix compile the data contained in the
            %       object in a matrix gm to be used in the standard mesh
            %       generation functions of the pde toolbox.
         
                 
                 
            
            n=size(nodes,2);
            obj.nodes=nodes; 
            switch nargin 
                case 1
                   obj.edges=[1:n;[2:n 1]]; 
                   obj.types=ones(1,n);
                   obj.parameters=[ones(1,n);zeros(6,n)];
                       
                case 2
                   obj.edges=edges;
                   obj.types=ones(1,n);
                   obj.parameters=[ones(1,n);zeros(6,n)];
                    
                otherwise
                    obj.edges=edges;
                    obj.types=type;
                    obj.parameters=param;
            end  
        end
        
        function disp(obj)
            disp('Boundary object')
            disp('Nodes :')
            disp(obj.nodes)
            disp(' ')
            disp('Edges :')
            disp(obj.edges)
            disp(' ')
            disp('Types :')
            disp(obj.types)
            disp(' ')
            disp('Parameters :')
            disp(obj.parameters)
            disp(' ')  
        end       
        
        function GM=geometrymatrix(obj)
            n=size(obj.nodes,2);
            GM=zeros(12,n);
            GM(1,obj.types==1)=2;
            GM(1,obj.types==2)=1;
            GM(1,obj.types==3)=4;
            GM(2:3,:)=[obj.nodes(1,obj.edges(1,:));obj.nodes(1,obj.edges(2,:))];
            GM(4:5,:)=[obj.nodes(2,obj.edges(1,:));obj.nodes(2,obj.edges(2,:))];
            GM(6:12,:)=obj.parameters;
        end
        
        function plot(obj)
            GM=obj.geometrymatrix;
            pdegplot(GM)
            axis equal
        end
        
    end


end