classdef Mesh
    
    
    properties
        
        nodes
        edges
        triangles
        stats
        
    end
    
    
    methods
        
        function obj=Mesh(boundary,iter)
            % MESH : Mesh class. (version beta)
            %
            %
            %       Constructors :
            %
            %       obj=Mesh(boundary,iter) builds a mesh object in the
            %       domain define by the Boundary object boundary. iter is
            %       the number of call to the refinemesh function.
            %
            %
            %       Methods :
            %
            %       obj.disp displays some information about the Mesh
            %       object.
            %
            %       obj.plot draws the Mesh object.
            %
            %       un=obj.tri2nod(ut) interpolates a triangles data to a
            %       nodes data.
            %
            %       ut=obj.nod2tri(un) interpolates a nodes data to a
            %       triangles data.
            %
            %       obj.surf(u) plot a 3d surface representing u.
            %
            %       obj.contour(u) plot the contour lines representing u.
            %
            %       gu=obj.grad(u) computes the gradient of a function
            %       given on the nodes.
            %
            %       dU=obj.div(U) computes the divergence of the vector
            %       field U.
            %
            %       Lu=obj.laplace(u) computes the lapcacian of u.
            %
            %       i=obj.int(u) return the integral of u computed on the
            %       domain.
            %
            %       obj.quiver(F) plot the vector field given in F.
            %
            %       u=obj.solvepde(bcm,c,a,f) solve the pde
            %
            %           -div(c grad(u)) + au = f
            %
            %       using the boundary conditions described in the matrix
            %       bcm constructed by the function
            %       boundaryconditionmatrix.
            %
            %       [A,F]=obj.matrices(bcm,c,a,f) assembles the stiffness
            %       matrix and the source term to be solved to solve the
            %       pde problem.
            %
            %       [Nnodes,Ntriangles]=obj.neighbour Nnodes(i) is the
            %       list of neighbour nodes of i and Ntriangles(i) is the
            %       list of triangles touching i.
            %
            %       B=obj.order1(b1,b2) evaluate the part of the stiffness
            %       matrix coresponding to the term b.grad(u).
            %
            %       u=obj.solvepde2(bcm,c,b1,b2,a,f) solve the pde
            %
            %       -div(c grad(u)) + b.grad(u) + au = f
            %       using the boundary conditions described in the matrix
            %       bcm constructed by the function
            %       boundaryconditionmatrix.
            
            
            time=cputime;
            GM=boundary.geometrymatrix;
            [p,e,t]=initmesh(GM);
            for i = 1 : iter
                p=jigglemesh(p,e,t,'Iter',20);
                a=pdetrg(p,t);
                u=max(a)/min(a);
                if u>1000
                    it=find(a/min(a)>10);
                    size(it)
                    size(p)
                    [p,e,t]=refinemesh(GM,p,e,t,it');
                    size(p)
                else
                    [p,e,t]=refinemesh(GM,p,e,t);
                end
                p=jigglemesh(p,e,t,'Iter',20);
            end
            obj.nodes=p;
            obj.edges=e;
            obj.triangles=t;
            w=whos('obj');
            mem=w.bytes/1000000;
            q=mean(pdetriq(p,t));
            a=pdetrg(p,t);
            r=mean(a);
            rmin=min(a);
            rmax=max(a);
            time=cputime-time;
            obj.stats=[size(p,2) size(e,2) size(t,2) rmin rmax r rmax/rmin q mem time];   
        end
                 
        function disp(obj)
            disp('Mesh object')
            disp(['Nodes :              ' num2str(obj.stats(1))])
            disp(['Edges :              ' num2str(obj.stats(2))])
            disp(['Triangles :          ' num2str(obj.stats(3))])
            disp(['Min resolution :     ' num2str(obj.stats(4))])
            disp(['Max resolution :     ' num2str(obj.stats(5))])
            disp(['Mean resolution :    ' num2str(obj.stats(6))])
            disp(['Uniformity :         ' num2str(obj.stats(7))])
            disp(['Mean quality :       ' num2str(obj.stats(8))])
            disp(['Memory used(Mo) :    ' num2str(obj.stats(9))])
            disp(['Time(s) :            ' num2str(obj.stats(10))])
            disp(' ')
            
        end
        
         function m2=refine(obj,b,u,epsilon)
            time=cputime;
            gu=obj.grad(u);
            ngu=sqrt(gu(1,:).^2 + gu(2,:).^2);
            a=pdetrg(obj.nodes,obj.triangles);
            it=find(ngu.*sqrt(a)>epsilon);
            GM=b.geometrymatrix;
            [p,e,t]=refinemesh(GM,obj.nodes,obj.edges,obj.triangles,it');
            p=jigglemesh(p,e,t,'Iter',20);
            m2=Mesh(b,0);
            m2.nodes=p;
            m2.edges=e;
            m2.triangles=t;
            w=whos('m2');
            mem=w.bytes/1000000;
            q=mean(pdetriq(p,t));
            a=pdetrg(p,t);
            r=mean(a);
            rmin=min(a);
            rmax=max(a);
            time=cputime-time;
            m2.stats=[size(p,2) size(e,2) size(t,2) rmin rmax r rmax/rmin q mem time];
        end
        
        
        function plot(obj)
            pdemesh(obj.nodes,obj.edges,obj.triangles);
        end
        
        function un=tri2nod(obj,ut)
            un=pdeprtni(obj.nodes,obj.triangles,ut);
        end
        
        function ut=nod2tri(obj,un)
            ut=pdeintrp(obj.nodes,obj.triangles,un);
        end
        
        function un=grid2mesh(obj,X,Y,Z)
            p=obj.nodes;
            un=interp2(X,Y,Z,p(1,:),p(2,:))';
        end
        
        function Z=mesh2grid(obj,x,y,u)
            Z=tri2grid(obj.nodes,obj.triangles,u,x,y);
        end
            
        function surf(obj,u)
            pdesurf(obj.nodes,obj.triangles,u);
            view([0 90])
            axis equal
        end
        
        function contour(obj,u)
            p=obj.nodes;
            t=obj.triangles;
            pdecont(obj.nodes,obj.triangles,u);
            axis equal
            colormap default
            view([0 90])
            
        end
        
        function gu=grad(obj,u)
            [gu1,gu2]=pdegrad(obj.nodes,obj.triangles,u);
            gu=[gu1;gu2];
        end
        
        function dU=div(obj,U)
            [gu11,gu12]=pdegrad(obj.nodes,obj.triangles,U(1,:)');
            [gu21,gu22]=pdegrad(obj.nodes,obj.triangles,U(2,:)');
            dU=gu11+gu22;
        end
        
        function Lu=laplace(obj,u)
            [gu1,gu2]=pdegrad(obj.nodes,obj.triangles,u);
            gu1=obj.tri2nod(gu1);
            gu2=obj.tri2nod(gu2);
            [ggu11,ggu12]=pdegrad(obj.nodes,obj.triangles,gu1);
            [ggu21,ggu22]=pdegrad(obj.nodes,obj.triangles,gu2);
            Lu=ggu11+ggu22;
            
            
        end
        
        function i=int(obj,u)
            p=obj.nodes;
            t=obj.triangles;
            a=pdetrg(p,t);
            if length(u)==length(p(1,:))
                ut=obj.nod2tri(u);
            else
                ut=u;
            end
            i=sum(ut.*a);
        end
        
        function quiver(obj,F)
            p=obj.nodes;
            if size(p,2)~=size(F,2)
                F1=obj.tri2nod(F(1,:))';
                F2=obj.tri2nod(F(2,:))';
            else
                F1=F(1,:);
                F2=F(2,:);
            end
            size(p)
            size(F1)
            quiver(p(1,:),p(2,:),F1,F2);
           
            axis equal
            
        end
        
        function u=solvepde(obj,BCM,c,a,f)
            u=assempde(BCM,obj.nodes,obj.edges,obj.triangles,c,a,f);
        end
        
        function [A,F]=matrices(obj,BCM,c,a,f)
            [A,F]=assempde(BCM,obj.nodes,obj.edges,obj.triangles,c,a,f);
        end
        
        function [Nnodes,Ntriangles]=neighbour(obj)
            np=obj.stats(1);
            Nnodes=zeros(np,10);
            Ntriangles=zeros(np,10);
            T=obj.triangles;
            nt=obj.stats(3);
            for k = 1 : nt
                t=T(:,k);
                v=union(Nnodes(t(1),:),t);
                Nnodes(t(1),1:length(v)-1)=v(2:end);
                v=union(Nnodes(t(2),:),t);
                Nnodes(t(2),1:length(v)-1)=v(2:end);
                v=union(Nnodes(t(3),:),t);
                Nnodes(t(3),1:length(v)-1)=v(2:end);
                
                v=union(Ntriangles(t(1),:),k);
                Ntriangles(t(1),1:length(v)-1)=v(2:end);
                v=union(Ntriangles(t(2),:),k);
                Ntriangles(t(2),1:length(v)-1)=v(2:end);
                v=union(Ntriangles(t(3),:),k);
                Ntriangles(t(3),1:length(v)-1)=v(2:end);
            end
        end
        
        function B=order1(obj,b1,b2)
            p=obj.nodes;
            t=obj.triangles;
            np=obj.stats(1);
            
            if  ischar(b1)
                b1=[b1 '+0*x'];
                x=p(1,:);
                y=p(2,:);
                bb1=obj.nod2tri(eval(b1)');
            elseif length(b1)==1
                bb1=b1*ones(1,np);
            else
                bb1=b1;
            end
            if  ischar(b2)
                b1=[b1 '+0*x'];
                x=p(1,:);
                y=p(2,:);
                bb2=obj.nod2tri(eval(b2)');
            elseif length(b2)==1
                bb2=b2*ones(1,np);
            else
                bb2=b2;
            end
            b=[bb1;bb2];


            [Nn,Nt]=obj.neighbour;
            B=sparse(np,np);
            parfor i = 1 : np
               Line=sparse(1,np);
               Ti=Nt(i,:);
               Ti=Ti(Ti>0);
               Vn=Nn(i,:);
               Vn=Vn(Vn>0);
               for k = 1 : length(Vn)
                   j=Vn(k);
                   Tj=Nt(j,:);
                   Tj=Tj(Tj>0);
                   Tcommun=intersect(Ti,Tj);
                   Line(j)=coeforder1(i,j,p,t,Tcommun,b);
               end
               B(i,:)=Line;
            end
        end
        
        function u=solvepde2(obj,BCM,c,b1,b2,a,f)
            [A,F]=obj.matrices(BCM,c,a,f);
            B=obj.order1(b1,b2);
            u=(A+B)\F;
        end
 
    end
    
    
end
